require File.expand_path(File.dirname(__FILE__) + '/spec_helper')

describe "BenchmarkHarness" do
  after :each do
    BenchmarkHarness.flush!
  end
  
  # #measure used in StringExample
  describe "#measure" do
    it "should add each instance of a run to a named collection" do
      2.times{
        StringExample.concatenate_with_plus("Hello ", "Dave")
        StringExample.concatenate_with_append("Hello ", "Dave")
      }
      BenchmarkHarness.collections[:plus].size.should == 2
      BenchmarkHarness.collections[:append].size.should == 2
    end
  end
  
  describe "Collection" do
    it "should provide a mean, median, mode, range, standard deviation, and variance for each kind of time if no key provided" do
      Benchmark::Tms.any_instance.stubs(:to_a).returns((1..6).to_a)
      2.times{
        StringExample.concatenate_with_plus("Hello ", "Dave")
      }
    
      BenchmarkHarness.collections[:plus].mean().should == {
        :user            => 2,
        :system          => 3,
        :children_user   => 4,
        :children_system => 5,
        :real            => 6
      }
      
      BenchmarkHarness.collections[:plus].median().should == {
        :user            => 2,
        :system          => 3,
        :children_user   => 4,
        :children_system => 5,
        :real            => 6
      }
      
      BenchmarkHarness.collections[:plus].mode().should == {
        :user            => 2,
        :system          => 3,
        :children_user   => 4,
        :children_system => 5,
        :real            => 6
      }
      
      BenchmarkHarness.collections[:plus].range().should == {
        :user            => 0,
        :system          => 0,
        :children_user   => 0,
        :children_system => 0,
        :real            => 0
      }
      
      BenchmarkHarness.collections[:plus].standard_deviation().should == {
        :user            => 0,
        :system          => 0,
        :children_user   => 0,
        :children_system => 0,
        :real            => 0
      }
      
      BenchmarkHarness.collections[:plus].variance().should == {
        :user            => 0,
        :system          => 0,
        :children_user   => 0,
        :children_system => 0,
        :real            => 0
      }
    end
    
    it "should provide *only* the mean, median, mode, range, etc when key is given" do
      Benchmark::Tms.any_instance.stubs(:to_a).returns((1..6).to_a)
      2.times{
        StringExample.concatenate_with_plus("Hello ", "Dave")
      }
    
      BenchmarkHarness.collections[:plus].mean(:user).should == 2
    end
  end
  
  describe "#wrap" do
    it "should wrap a class method with benchmark harness measure" do
      BenchmarkHarness.wrap_method("NestedExample::OneLevelDown.add", :add)
      NestedExample::OneLevelDown.add
      BenchmarkHarness.collections[:add].should_not be_empty      
    end
    
    it "should wrap an instance method with benchmark harness measure" do
      BenchmarkHarness.wrap_method("NestedExample::OneLevelDown#subtract", :subtract)
      NestedExample::OneLevelDown.new.subtract
      BenchmarkHarness.collections[:subtract].should_not be_empty
    end
    
    it "should wrap a method with an '!' in the name" do
      BenchmarkHarness.wrap_method("NestedExample#important!", :important)
      NestedExample.new.important!
      BenchmarkHarness.collections[:important].should_not be_empty
    end
  end
  
  describe "#flush" do
    it "should remove all previous samples" do
      2.times{
        StringExample.concatenate_with_plus("Hello ", "Dave")
      }
      BenchmarkHarness.flush!
    
      BenchmarkHarness.collections.should be_empty
    end
  end
  
  describe "#print" do
    it "should print without throwing an error" do
      2.times{
        StringExample.concatenate_with_plus("Hello ", "Dave")
      }    
      lambda{BenchmarkHarness.collections[:plus].print}.should_not raise_error
      
    end
  end
end
